#include <stdio.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <stdlib.h>
#include <string.h>

/* register settings default configuration */
#define MAIN_PARAMETER       0x11 
#define FREQ2A_PARAMETER     0xD6 
#define FREQ1A_PARAMETER     0x40
#define FREQ0A_PARAMETER     0x00
#define FREQ2B_PARAMETER     0xD6
#define FREQ1B_PARAMETER     0x47
#define FREQ0B_PARAMETER     0x2B

#define FSEP1_PARAMETER      0x03
#define FSEP0_PARAMETER      0xE3

#define RX_CURRENT_PARAMETER 0x8F
#define FRONT_END_PARAMETER  0x30
#define PA_POW_PARAMETER     0xFF
#define PLL_RX_PARAMETER     0x70
#define LOCK_PARAMETER       0x10
#define CAL_PARAMETER        0x26
#define MODEM2_PARAMETER     0x90
#define MODEM1_PARAMETER     0x6F

#define MODEM0_PARAMETER     0x45 	//9600 baudrate

#define MATCH_PARAMETER      0x20
#define FSCTRL_PARAMETER     0x01
#define FSHAPE7_PARAMETER    0x00
#define FSHAPE6_PARAMETER    0x00
#define FSHAPE5_PARAMETER    0x00
#define FSHAPE4_PARAMETER    0x00
#define FSHAPE3_PARAMETER    0x00
#define FSHAPE2_PARAMETER    0x00
#define FSHAPE1_PARAMETER    0x00
#define FSDELAY_PARAMETER    0x00
#define PRESCALER_PARAMETER  0x00
#define TEST6_PARAMETER      0x00
#define TEST5_PARAMETER      0x00
#define TEST4_PARAMETER      0x3F
#define TEST3_PARAMETER      0x00
#define TEST2_PARAMETER      0x00
#define TEST1_PARAMETER      0x00
#define TEST0_PARAMETER      0x00
#define TX_CURRENT_PARAMETER 0xF3
#define PLL_TX_PARAMETER     0x70

/*definition of CC1000 registers*/
#define CC1000_MAIN            0x00
#define CC1000_FREQ_2A         0x01
#define CC1000_FREQ_1A         0x02
#define CC1000_FREQ_0A         0x03
#define CC1000_FREQ_2B         0x04
#define CC1000_FREQ_1B         0x05
#define CC1000_FREQ_0B         0x06
#define CC1000_FSEP1           0x07
#define CC1000_FSEP0           0x08
#define CC1000_CURRENT         0x09
#define CC1000_FRONT_END       0x0A
#define CC1000_PA_POW          0x0B
#define CC1000_PLL             0x0C
#define CC1000_LOCK            0x0D
#define CC1000_CAL             0x0E
#define CC1000_MODEM2          0x0F
#define CC1000_MODEM1          0x10
#define CC1000_MODEM0          0x11
#define CC1000_MATCH           0x12
#define CC1000_FSCTRL          0x13
#define CC1000_FSHAPE7         0x14
#define CC1000_FSHAPE6         0x15
#define CC1000_FSHAPE5         0x16
#define CC1000_FSHAPE4         0x17
#define CC1000_FSHAPE3         0x18
#define CC1000_FSHAPE2         0x19
#define CC1000_FSHAPE1         0x1A		
#define CC1000_FSDELAY         0x1B		
#define CC1000_PRESCALER       0x1C		
#define CC1000_TEST6           0x40
#define CC1000_TEST5           0x41
#define CC1000_TEST4           0x42
#define CC1000_TEST3           0x43
#define CC1000_TEST2           0x44
#define CC1000_TEST1           0x45
#define CC1000_TEST0           0x46

/*the bit definitions of the MAIN register*/
#define CC1000_RXTX			7
#define CC1000_F_REG		6
#define CC1000_RX_PD		5
#define CC1000_TX_PD		4
#define CC1000_FS_PD		3
#define CC1000_CORE_PD		2
#define CC1000_BIAS_PD		1
#define CC1000_RESET_N		0

/*the bit definitions in the CURRENT register*/
//CURRENT[7-4] control of current in VCO core for TX and RX
#define CC1000_VCO_CURRENT	4
//control of current in VCO buffer for LO drive
#define CC1000_LO_DRIVE		2
//control of current in VCO buffer for PA
#define CC1000_PA_DRIVE		0

/*the bit definitions in the FRONT_END register*/
#define CC1000_BUF_CURRENT	5
#define CC1000_LNA_CURRENT	3
#define CC1000_IF_RSSI		1
#define CC1000_XOSC_BYPASS	0

/*the bit definitions in the PLL register*/
#define CC1000_EXT_FILTER	7
#define CC1000_REFDIV		3
#define CC1000_ALARM_DISABLE	2
#define CC1000_ALARM_H		1
#define CC1000_ALARM_L		0

/*the bit definitions in the LOCK register*/
#define CC1000_LOCK_SELECT	4
#define CC1000_PLL_LOCK_ACCURACY	3
#define CC1000_PLL_LOCK_LENGTH	2
#define CC1000_LOCK_INSTANT	1
#define CC1000_LOCK_CONTINUOUS	0

/*bit definitions in the CAL register*/
#define CC1000_CAL_START	7
#define CC1000_CAL_DUAL		6
#define CC1000_CAL_WAIT		5
#define CC1000_CAL_CURRENT	4
#define CC1000_CAL_COMPLETE	3
#define CC1000_CAL_ITERATE	0  //3 bits

/*bit definitions in the MODEM2 register*/
#define CC1000_PEAKDETECT		7 
#define CC1000_PEAK_LEVEL_OFFSET	0 //7 bits

/*bit definitions in MODEM1 register*/ 
#define CC1000_MLIMIT		5
#define CC1000_LOCK_AVG_IN	4
#define CC1000_LOCK_AVG_MODE	3
#define CC1000_SETTLING		1
#define CC1000_MODEM_RESET_N	0

// Bit Definitions in  MODEM0 Register 
#define CC1000_BAUDRATE		4
#define CC1000_DATA_FORMAT	2
#define CC1000_XOSC_FREQ	0

/*/bit definitions in MATCH register 
#define CC1000_RX_MATCH		4
#define CC1000_TX_MATCH		0
*/
/*bit definitions in FSCTLR register 
#define CC1000_DITHER1		3
#define CC1000_DITHER0		2
#define CC1000_SHAPE		1
#define CC1000_FS_RESET_N	0
*/
/*bit definitions in PRESCALER register 
#define CC1000_PRE_SWING		6
#define CC1000_PRE_CURRENT	4
#define CC1000_IF_INPUT		3
#define CC1000_IF_FRONT		2
*/
/*bit Definitions in TEST6 Register
#define CC1000_LOOPFILTER_TP1	7
#define CC1000_LOOPFILTER_TP2	6
#define CC1000_CHP_OVERRIDE	5
#define CC1000_CHP_CO		0
*/

/*bit definitions in TEST5 register*/ 
#define CC1000_CHP_DISALBE	5
#define CC1000_VCO_OVERRIDE	4
#define CC1000_VCO_AO		0

/*definition of pins used to configuraton the CC1000*/
#define CONFIG_PORT      PORTD
#define CONFIG_PORT_DIR  DDRD
#define CONFIG_PORT_PIN  PIND
#define PALE             4
#define PDATA            7
#define PCLK             6

/*definition of pins used to data communication*/   
#define DATA_PORT PORTB
#define DATA_PIN PINB
#define DATA_DDR DDRB
#define DCLK 1
#define DIO 3

/*this is master "chip select" pin*/
#define DDRCC 0	  //SS
#define DDRCC_PORT PORTB

/*mode definitions*/
#define CC1000_MODE_RX    0x00
#define CC1000_MODE_TX    0x01
#define CC1000_MODE_PD    0x02
#define CC1000_MODE_SLEEP 0x03




