#include "cc1000_routines.h"


// Global variables

/* Current frequency. */
uint8_t cur_freq;

/* Current power level. */
uint8_t rf_power;

/* cc1000_routines _________________________________________________________*/

void cc1000_reset (void)
{
   uint8_t mainValue;

   mainValue = cc1000_read(CC1000_MAIN);//Reset CC1000 Bit
   cc1000_write(CC1000_MAIN, mainValue & 0xFE);
   cc1000_write(CC1000_MAIN, mainValue | 0x01); //Set Reset Bit to 1
}

void cc1000_calibrate (void)
{
   cc1000_write(CC1000_PA_POW, 0x00);   // turn off radio power
   cc1000_write(CC1000_TEST4, TEST4_PARAMETER);

   cc1000_write(CC1000_MAIN, ((1 << CC1000_TX_PD) | (1 << CC1000_RESET_N)));

   //  RX current  is the VCO current to be used in RX mode
   cc1000_write(CC1000_CURRENT, RX_CURRENT_PARAMETER);

   cc1000_write(CC1000_CAL, ((1 << CC1000_CAL_START) |
                             (1 << CC1000_CAL_WAIT) |
                             (6 << CC1000_CAL_ITERATE)));

   while((cc1000_read(CC1000_CAL) & (1 << CC1000_CAL_COMPLETE)) == 0);

   cc1000_write(CC1000_CAL, ((1 << CC1000_CAL_WAIT) |
                             (6 << CC1000_CAL_ITERATE)));


   cc1000_write(CC1000_MAIN, ((1 << CC1000_RXTX) |
                              (1 << CC1000_F_REG) |
                              (1 << CC1000_RX_PD) | (1 << CC1000_RESET_N)));

     cc1000_write(CC1000_CURRENT, TX_CURRENT_PARAMETER);

   //PA is turned off to prevent spurious emission
   cc1000_write(CC1000_PA_POW, 0x00);

   // Write CAL: CAL_START=1
   cc1000_write(CC1000_CAL, ((1 << CC1000_CAL_START) |
                             (1 << CC1000_CAL_WAIT) |
                             (6 << CC1000_CAL_ITERATE)));

   while((cc1000_read(CC1000_CAL) & (1 << CC1000_CAL_COMPLETE)) == 0);

   // Write CAL: CAL_START=0
   cc1000_write(CC1000_CAL, ((1 << CC1000_CAL_WAIT) |
                              (6 << CC1000_CAL_ITERATE)));
}

void cc1000_set_power(uint8_t power)
{
   rf_power = power;
   cc1000_write(CC1000_PA_POW, power);
   delay(1);
}

void cc1000_mode (uint8_t mode)
{
   switch (mode) {
   
   case CC1000_MODE_RX:
 
      cc1000_write(CC1000_MAIN, ((1 << CC1000_TX_PD) | (1 << CC1000_RESET_N)));
      cc1000_write(CC1000_PA_POW, 0);
      cc1000_write(CC1000_CURRENT, RX_CURRENT_PARAMETER);
      delay(1);	// Wait 250 us
      break;

   case CC1000_MODE_TX:

      cc1000_write(CC1000_MAIN, ((1 << CC1000_RXTX) |
                                 (1 << CC1000_F_REG)|
                                 (1 << CC1000_RX_PD)| 
								 (1 << CC1000_RESET_N)));
      cc1000_write(CC1000_CURRENT, TX_CURRENT_PARAMETER);
      delay(1);	// Wait 250 us
      cc1000_write(CC1000_PA_POW, rf_power);
      delay(8/100);	// Wait 20 us
      break;

   case CC1000_MODE_PD:
      cc1000_write(CC1000_MAIN, 0x3F); 
      cc1000_write(CC1000_PA_POW, 0x00);
      break;

   case CC1000_MODE_SLEEP:
      cc1000_write(CC1000_MAIN, ((1 << CC1000_RX_PD) |
                                 (1 << CC1000_TX_PD) |
                                 (1 << CC1000_FS_PD) |
                                 (1 << CC1000_CORE_PD) |
                                 (1 << CC1000_BIAS_PD) |
                                 (1 << CC1000_RESET_N)));
      break;
   }
}

void rssi_on (void)
{
   cc1000_write(CC1000_FRONT_END, 0x32);
}

void rssi_off (void)
{
   cc1000_write(CC1000_FRONT_END, 0x30);
}

void cc1000_on (void)
{
   cc1000_write(CC1000_MAIN, ((1 << CC1000_RX_PD) |
                              (1 << CC1000_TX_PD) |
                              (1 << CC1000_FS_PD) |
                              (1 << CC1000_BIAS_PD) | (1 << CC1000_RESET_N)));

   delay(4);
   cc1000_mode(CC1000_MODE_RX);
}

void cc1000_wakeup (void)
{
   cc1000_on();
   delay(2);
}

/***************INIT CC1000*************************************************/

void cc1000_set (uint8_t new_freq)
{

   // record the current setting frequency    
   cur_freq = new_freq;

   // record the current setting power
   rf_power = PA_POW_PARAMETER;
	

	cc1000_write(CC1000_MAIN, MAIN_PARAMETER);

	cc1000_write(CC1000_FREQ_2A, FREQ2A_PARAMETER);
	cc1000_write(CC1000_FREQ_1A, FREQ1A_PARAMETER);
	cc1000_write(CC1000_FREQ_0A, FREQ0A_PARAMETER);
	cc1000_write(CC1000_FREQ_2B, FREQ2B_PARAMETER);
	cc1000_write(CC1000_FREQ_1B, FREQ1B_PARAMETER);
	cc1000_write(CC1000_FREQ_0B, FREQ0B_PARAMETER);
	/*cc1000_write(CC1000_FREQ_2A, 0x75);
	cc1000_write(CC1000_FREQ_1A, 0xA0);
	cc1000_write(CC1000_FREQ_0A, 0x00);
	cc1000_write(CC1000_FREQ_2B, 0x58);
	cc1000_write(CC1000_FREQ_1B, 0x33);
	cc1000_write(CC1000_FREQ_0B, 0x13);*/
	/*cc1000_write(CC1000_FREQ_2A, 0xD8);
	cc1000_write(CC1000_FREQ_1A, 0xC0);
	cc1000_write(CC1000_FREQ_0A, 0x00);
	cc1000_write(CC1000_FREQ_2B, 0xD8);
	cc1000_write(CC1000_FREQ_1B, 0xC7);
	cc1000_write(CC1000_FREQ_0B, 0x2B);*/
   
   cc1000_write(CC1000_FSEP1, FSEP1_PARAMETER);
   cc1000_write(CC1000_FSEP0, FSEP0_PARAMETER);
   cc1000_write(CC1000_CURRENT, RX_CURRENT_PARAMETER);
   cc1000_write(CC1000_FRONT_END, FRONT_END_PARAMETER);
   cc1000_write(CC1000_PA_POW, PA_POW_PARAMETER);
   cc1000_write(CC1000_PLL, PLL_RX_PARAMETER);
   cc1000_write(CC1000_LOCK, LOCK_PARAMETER);
   cc1000_write(CC1000_CAL, CAL_PARAMETER);
   cc1000_write(CC1000_MODEM2, MODEM2_PARAMETER);
   cc1000_write(CC1000_MODEM1, MODEM1_PARAMETER);
   cc1000_write(CC1000_MODEM0, MODEM0_PARAMETER);
   cc1000_write(CC1000_MATCH, MATCH_PARAMETER);
   cc1000_write(CC1000_FSCTRL, FSCTRL_PARAMETER);
   cc1000_write(CC1000_FSHAPE7, FSHAPE7_PARAMETER);
   cc1000_write(CC1000_FSHAPE6, FSHAPE6_PARAMETER);
   cc1000_write(CC1000_FSHAPE5, FSHAPE5_PARAMETER);
   cc1000_write(CC1000_FSHAPE4, FSHAPE4_PARAMETER);
   cc1000_write(CC1000_FSHAPE3, FSHAPE3_PARAMETER);
   cc1000_write(CC1000_FSHAPE2, FSHAPE2_PARAMETER);
   cc1000_write(CC1000_FSHAPE1, FSHAPE1_PARAMETER);
   cc1000_write(CC1000_FSDELAY, FSDELAY_PARAMETER);
   cc1000_write(CC1000_TEST4, TEST4_PARAMETER);
   cc1000_write(CC1000_PRESCALER, PRESCALER_PARAMETER);

   //after setting new frequency recalibrate cc1000 
   cc1000_calibrate();
   //put cc1000 into receive mode
   cc1000_mode(CC1000_MODE_RX);
}

uint8_t cc1000_get_power (void)
{
   return rf_power;
}

void cc1000_init (uint8_t freq)
{
   cur_freq = freq; // record the current setting frequency

   // record the current setting power
    rf_power = 0xff; // max power

   CONFIG_PORT_DIR |= ((1 << PALE) | (1 << PCLK) | (1 << PDATA));

   // Set the three configuration pin high
   PALE_HIGH();
   PCLK_HIGH();
   PDATA_HIGH();
   
   //the crystal oscillator core will turn on
   cc1000_write(CC1000_MAIN, ((1 << CC1000_RX_PD) |
                              (1 << CC1000_TX_PD) |
                              (1 << CC1000_FS_PD) |
			      (1 << CC1000_BIAS_PD)));

   //reset
   cc1000_write(CC1000_MAIN, ((1 << CC1000_RX_PD)   |
                              (1 << CC1000_TX_PD)   |
                              (1 << CC1000_FS_PD)   |
                              (1 << CC1000_BIAS_PD) |
			      (1 << CC1000_RESET_N)));
   delay(8);	// wait 2 ms

   // Program all registers, Calibrate VCO and PLL and set the frequency
   cc1000_set(freq); //not used yet
   
   UART_Init(); // Setup default baud rate
   SPI_Init();	// Spi bus init 
}
