#include "cc1000_spi_state.h"

#define TX_PRE			0x01
#define TX_ACK			0x02
#define TX_CRC			0x03
#define TX_SIZE			0x04
#define TX_DATA			0x05
#define TX_DONE			0x06
#define TX_WAIT_FOR_ACK 0x07

#define RX_IDLE			0x08
#define RX_SYNC 		0x09
#define RX_ACK			0x10
#define RX_CRC			0x11
#define RX_SIZE			0x12
#define RX_DATA			0x13
#define RX_TX_ACK		0x14
#define RX_DONE			0x15


/* number preamble bytes you must receive*/
#define PREAMBLE_REC 16
/* numebr preamble bytes you have to send*/ 
#define PREAMBLE_SEND 32

#define ACK_BYTE 	0x66
#define SYNC_BYTE 	0x33
#define PREAMBLE 	0xAA
#define FLUSH_BYTE 	0xFF

volatile char state = RX_IDLE;

volatile uint8_t s, x, ack_count, payload_count, data_size;

uint16_t r_counter =0;


volatile static char size;
volatile char size_comp;
volatile char payload;	//receive data
volatile char data_rec;	//receiving
volatile char prev_data; //receiving
volatile char crc_; //compare crc_
volatile static char crc;	// receive crc
volatile char ack;	//receive ack

char tx_CRC; //transmit crc

volatile static uint8_t offset;
volatile static uint8_t preamble_count;

volatile char tx_data;
volatile char rx_data;

void cc1000_state();

char *send_ptr;	//pointer on tx_Buffer
char *rec_ptr;	//pointer on rx_Buffer
 
//char tx_Buffer[] = {1, 2, 3, 4, 5};//nicht

//-----------------------------------------------------------------------------

SIGNAL(SIG_SPI)
{	
	cc1000_state();
}

void cc1000_state()
{	
	switch(state)
	{
		/*TX_STATE_MACHINE*/
		case TX_PRE:
			
            if(++preamble_count <= PREAMBLE_SEND) {
				SPDR = PREAMBLE;
            } else {
				SPDR = SYNC_BYTE;
				state = TX_ACK;
			}
		break;
		
		case TX_ACK:
				
				SPDR = ACK_BYTE;
				state = TX_CRC;
		break;

		case TX_CRC:

				SPDR = tx_CRC;
				state = TX_SIZE;
				//for double size transmission		
		break;			

		case TX_SIZE:
				
				SPDR = data_size;
				state = TX_DATA;
		break;

		case TX_DATA:			
			
			if(TxR!=TxW){
			
    		SPDR = readFromTxBuffer(); //read next character from the buffer
			
			}else{
				
				state = TX_DONE;
			}
		break;

		case TX_DONE:
			
			SPDR = FLUSH_BYTE;
			ack_count=0;
			cc1000_mode (CC1000_MODE_RX);
			state = TX_WAIT_FOR_ACK;
		break;

		case TX_WAIT_FOR_ACK:
			
			rx_data = SPDR;
            for (offset = 0; offset < 8; offset++) {
       			if ((((prev_data << offset)|(rx_data >> (8 - offset))) & 0xff) == ACK_BYTE) {
      			no_ack=0;
				PORTA=0x05;
				p_counter++;
				state = RX_IDLE;
				break;
                }    
            }
            if(ack_count >= 12) {            
				no_ack=1;
				PORTA=0x0B;
				p_counter_false++;
				state = RX_IDLE;
				break;                
            }
            prev_data = rx_data;   
			ack_count++;		
		break;

		/*RX_STATE_MACHINE*/
		case RX_SYNC:
		
		offset = 0;	
		//rx_data=SPDR;
        while(prev_data != SYNC_BYTE && offset < 8)
		{
               prev_data = (prev_data << 1) | (SPDR >> (7 - offset));
               offset++;
        }
		    	     
			if(offset<8){		
				prev_data = SPDR;
				state = RX_ACK;
				
			}else{
                state = RX_IDLE;
            }
		
		break;
		
		case RX_ACK:
			
			//rx_data=SPDR;
			ack = (prev_data << offset) | (SPDR >> (8 - offset));
			//writeToRxBuffer(ack);
			
			prev_data = SPDR;//rx_data;
			state = RX_CRC;
		break;	

		case RX_CRC:

			//rx_data=SPDR;
			if((prev_data << offset) | (SPDR >> (8 - offset)) == 0xCD)
			{
			//crc = (prev_data << offset) | (SPDR >> (8 - offset));
			//writeToRxBuffer(crc);
				prev_data = SPDR;//rx_data;
				state = RX_SIZE;
			}else{
				state = RX_IDLE;
				preamble_count=0;
			}
		break;

		case RX_SIZE:
			
			//rx_data=SPDR;
			//if((prev_data << offset) | (SPDR >> (8 - offset)) == 0x05)
			//{
				size = (prev_data << offset) | (SPDR >> (8 - offset));
				//writeToRxBuffer(size);
				prev_data = SPDR;
				state = RX_DATA;
				payload_count = 0;
			/*}else{
				state = RX_IDLE;
				preamble_count=0;
			}*/
			// TODO send size two times 			
		break;

		case RX_DATA:
			if(buffer_overrun){ 
			state = RX_IDLE;
			preamble_count=0;
			PORTA=0xff;
			buffer_overrun=0;
			break;
			}

			//rx_data=SPDR;
			payload = (prev_data << offset) | (SPDR >> (8 - offset));
			writeToRxBuffer(payload);
			
			if(++payload_count == size){
				
					cc1000_mode (CC1000_MODE_TX);
					state = RX_TX_ACK;
					ack_count = 0;
			}
			prev_data = SPDR;		
		break;

		case RX_TX_ACK:
			PORTA=0x00;
			if (ack_count < 4) {
                SPDR = PREAMBLE;
                ack_count++;
            } else if (ack_count < 6) {
                SPDR = ack;
                ack_count++;
            } else {
                SPDR = FLUSH_BYTE;
                state = RX_IDLE;
				cc1000_mode (CC1000_MODE_RX);
				PORTA=0xFF;
				r_counter++;
				dataPacketReady=1;	
            }   

		break;

		/*default state*/
		case RX_IDLE:
		default:
			dataPacketReady=0;
			rx_data=SPDR;
			if(rx_data == (char)0xAA || rx_data == (char)0x55) 
			{
				preamble_count++;
			}
			
			else {
				
				if(preamble_count >= PREAMBLE_REC)
				{						                              
			        /* the MSB of the sync byte might be the LSB of the 
			        last preamble byte => offset = 7 and no sync state */                              
			        if(((prev_data << 7) | (rx_data >> 1)) == SYNC_BYTE) {
			        	offset = 7;
			        	state = RX_DATA;
			       	} else
			        	state = RX_SYNC;
				}
				preamble_count = 0;
			}
		prev_data=rx_data;		            
		break;
	}
	
}

void DoTx(void)
{
	cli();
	uint8_t i, x = 0x01;
	for(i=0;i<5;i++){
		writeToTxBuffer(x);
		crc_Buffer[i] = x;
		x++;
	}
	
	no_ack=1;

	/* check the air for transmission */
	//while(check_noise_value()==0) PORTA=0x06;
	//PORTA=0xFF;
	cc1000_mode(CC1000_MODE_TX);
	state = TX_PRE;
	
	/* tx_Buffer content size */
	data_size = crc_bufferContentSize()/8;
	
	/* do not compute crc inside of Interrupt */
	send_ptr = crc_Buffer; 
	tx_CRC = 0xCD;//update_crc(crc_Buffer, send_ptr);

	sei();
	/* start to send the first preamble byte*/
	preamble_count = 1;
	SPDR = PREAMBLE;

	//p_counter++;
}

/* RX Routine */
void DoRx(void)
{
	char my_byte;
	
	/*if(ready_to_crc){
		ready_to_crc=0;
		rec_ptr = rx_Buffer;
		crc_ = update_crc(rx_Buffer, rec_ptr);
		if(crc==crc_){
		crc_ready=1;
		}
	}*/

	if(dataPacketReady){
	
	UART_SendByte(r_counter);
			while(RxR!=RxW){
			cli();
			my_byte = readFromRxBuffer();
			sei();
			//UART_SendByte(my_byte);
				/*if(my_byte == 3){
				r_counter5++;
				UART_SendByte(r_counter5);
				}else if(my_byte == 4){
				r_counter4++;
				UART_SendByte(r_counter4);
				}*/
			}
	}
}

char get_cc1000_state(void)
{
	return state;
}

void transmission(void)
{
	DoTx();
	while(get_cc1000_state()!=RX_IDLE);
	if(no_ack==1){
		PORTA=0x0B;
	 	transmission();
	 	//UART_SendByte(0x00);
	}else if(no_ack==0){
		PORTA=0x05;
		//UART_SendByte(0x01);
		check_noise_value();
	}
}


void b_mac_transmission(void)
{
	/* if we arent sending or receiving any valid data 
	try to update the noise floor*/
	if(state==RX_IDLE);
	check_noise_value();

	while(check_noise_value()==0) PORTA=0x06;
	PORTA=0xFF;
	transmission();
}
