#include "cc1000_wr_register.h"


/*time delay*/
void delay(int time) { 		//delay time * 0.250ms
  TCCR0 = 3;				//Prescale by 64 (2 Takte)
  while(time) {		
    TCNT0 = 0;				//Reset counter
    while(TCNT0<39);		//Wait
    
	// 39*64=2'496   10'000'000/2'496=4000
    // 1/4006.41=0.0002496s=~250us
    
	time--;
  }
  TCCR0 = 0;				//Stop
}


/* Write_to_register */
void cc1000_write (uint8_t addr, uint8_t val)
{
   uint8_t i;

   addr <<= 1;                  // We only want the lower 7 bits   
   PALE_LOW();

   // Send 7 address bits 
   for(i = 0; i < 7; i++) {
      if(addr & 0x80)
         PDATA_HIGH();   
      else
         PDATA_LOW(); 

      /* Now cycle PCLK to write the bit. */
      PCLK_LOW();  
      PCLK_HIGH(); 
      addr <<= 1;
   }

   // Send write bit 
   PDATA_HIGH();
   PCLK_LOW();
   PCLK_HIGH();
   PALE_HIGH(); 

   // Send 8 data bits    
   for(i = 0; i < 8; i++) {
      if(val & 0x80)
         PDATA_HIGH();   
      else
         PDATA_LOW();

      // Now cycle PCLK to write the bit
      PCLK_LOW(); 
      PCLK_HIGH(); 
      val <<= 1;
   }

   // Set the three configuration pins high 
   PALE_HIGH();
   PDATA_HIGH();
   PCLK_HIGH();
}

/* Read_from_register */
uint8_t cc1000_read (uint8_t addr)
{
   uint8_t i;
   uint8_t val = 0;

   PALE_HIGH();
   
   addr <<= 1; // We only want the lower 7 bits
   PALE_LOW(); 

   // Send 7 address bits
   for(i = 0; i < 7; i++) {
      PCLK_HIGH();
      
      if(addr & 0x80)
         PDATA_HIGH();  
      else
         PDATA_LOW(); 

      addr <<= 1;

      // Now cycle PCLK to write the bit
      PCLK_LOW();     
   }

   // Send read bit
   PCLK_HIGH();
   PDATA_LOW();       
   PCLK_LOW();
   
   PCLK_HIGH();
   PALE_HIGH(); 
   PDATA_HIGH();
   
   CONFIG_PORT_DIR &= ~(1 << PDATA); // Set up PDATA as an input
   
   // Receive data bits
   for(i = 0; i < 8; i++) {
      PCLK_LOW();
      
      if(CONFIG_PORT_PIN & (1 << PDATA))
         val = (val << 1) | 0x01;
      else
         val = (val << 1) & 0xFE;

      PCLK_HIGH();     
   }
   
   PALE_HIGH();
   CONFIG_PORT_DIR |= (1 << PDATA);     // Set up PDATA as an output again
   //CONFIG_PORT |= (1 << PDATA);
   PDATA_HIGH();

   return val;
}
