#include "crc.h"

/*****************************************************************************
* Name: crcCCITT()
* Eingaben: - Laenge der Nachricht
* - Zeiger auf die Nachricht
* - CRC-Vorladewert, Startwert ist 0xFFFF
* Ausgaben: - CRC von der Nachricht
* Beschreibung: Die Funktion berechnet den CRC von der Nachricht nach
dem
* Generatorpolynom g(x) = x^16+ x^12 + x^5 + 1 (CCITT-Polynom).
*****************************************************************************/


unsigned short crcCCITT(unsigned int size, unsigned char * mem, unsigned short crc)
{

//  const unsigned short poly = 0x1021; // g(x) = (x^16+) x^12 + x^5 + 1
#define poly  0x1021    // g(x) = (x^16+) x^12 + x^5 + 1
//  static int i,j;
  unsigned int i;
  unsigned char j;

//  static unsigned short b;
  unsigned char b;

//  static unsigned char m;
  unsigned char m;

  // While (more message bits)
  for (i=0; i < size; i++)
  {
    m = mem[i];
//    for (j=0; j < 8; j++)
    for( j = 8; j; j-- )
    {
//      b = (crc & 0x8000);
      b = 0;
      if(crc & 0x8000)
        b++;
      // Shift the register left by one bit,
      crc <<= 1;
      if ((m & 0x80) != 0) crc |= 1;
      m <<= 1;
      // If (a 1 bit popped out of the register during step 3)
            // Register = Register XOR Poly.
      if (b != 0) crc ^= poly;
    }
  }
  return crc;
}

/* easy crc-computetion*/
unsigned short CalcCrc16(unsigned char* a_pArr, const unsigned int
a_nLen) {
  unsigned short nCrc = 0, nIndex, nCount;
  for (nCount = 0; nCount < a_nLen; nCount++) {
    nCrc = nCrc ^ (a_pArr[nCount] << 8);
    for (nIndex = 1; nIndex <= 8; nIndex++) {
     if ((nCrc & 0x8000) != 0)
  nCrc = ((nCrc << 1) ^ 0x1021);
     else
  nCrc = (nCrc << 1);
     }
  }
  return (nCrc & 0xFFFF);
}

/* Update 8-bit CRC value
  using polynomial  X^8 + X^5 + X^4 + 1 */

#define POLYVAL 0x8C

unsigned char update_crc(unsigned char new_, unsigned char *crc)
{
  unsigned char c, i;

  c = *crc;
  for (i = 0; i < 8; i++) {
     if ((c ^ new_) & 1) c = (c >> 1 ) ^ POLYVAL;
     else c >>= 1;
     new_ >>= 1;
  }
 *crc = c;
 return c;
}
