#include <stdio.h>
#include <stdlib.h>
#include <avr/io.h>
#include <avr/signal.h>
#include <avr/interrupt.h>
#include <avr/delay.h>

#define BAUD 9600L          // Baudrate, das L am Ende ist wichtig, NICHT UL verwenden!

// Berechnungen
#define UBRR_VAL ((F_CPU+BAUD*8)/(BAUD*16)-1)   // clever runden
#define BAUD_REAL (F_CPU/(16*(UBRR_VAL+1)))     // Reale Baudrate
#define BAUD_ERROR ((BAUD_REAL*1000)/BAUD-1000) // Fehler in Promille 

#if ((BAUD_ERROR>10) || (BAUD_ERROR<-10))
#error Systematischer Fehler der Baudrate grösser 1% und damit zu hoch! 
#endif


#define _UDR    UDR0
#define _UCSRA UCSR0A
#define _UCSRB UCSR0B
#define _UCSRC UCSR0C
#define _UBRRH UBRR0H 
#define _UBRRL UBRR0L 

volatile uint8_t send;

typedef struct TestStruct{
    uint32_t i;
}TestStruct;

TestStruct tvar;


SIGNAL(SIG_UART0_TRANS) {
	PORTA = 0xFF-4;
}

SIGNAL(SIG_UART0_RECV) {
	PORTA = 0xFF-4;
}

#define busy_loop_until(x) while(!(x)) {}
int uart_putc(unsigned char c){
/*
	PORTA = 0xFF-4;
	busy_loop_until(!(_UCSRA & (1<<UDRE)));
	PORTA = 0xFF-2;
*/
	//busy_loop_until(!send);
	_UDR = c;
	_UCSRA |= (1<<TXC);
	return 0;
}

/* puts ist unabhaengig vom Controllertyp */
void uart_puts (char *s){
	while (*s){   
		/* so lange *s != '\0' also ungleich dem "String-Endezeichen" */
		uart_putc(*s);
		s++;
		_delay_ms(1);
	}
}

/* puts ist unabhaengig vom Controllertyp */
void uart_puti (int32_t i){
    char s[7] ;
    itoa(i,s,10);
    uart_puts(s);
}


void uart_init(uint16_t ubbr) {
	_UBRRH = ubbr >> 8;
	_UBRRL = ubbr & 0xFF;

	_UCSRA = 0;
	_UCSRB |= (1<<TXEN) | (1<<RXEN); // UART TX einschalten
	_UCSRB |= (1<<TXCIE);		 // UART TX interrupt einschalten
	_UCSRB |= (1<<RXCIE);		 // UART RX interrupt einschalten
	sei();

	_UCSRC |= (1<<UCSZ1)|(1<<UCSZ0);     // Asynchron 8N1 
}

int main() {
    DDRA=0xff;
    PORTA=0x00;
	int ubbr=46;
	uart_init(ubbr);
    while(1) uart_puts("Hello, World!\n");
	return 0;
}

