#include "uart.h"

// UART global variables
         
volatile unsigned char Transmitter_Ready;

/* use UART0 in ATmega128 */

 #define UDR_ UDR0

 #define TXCIE_ TXCIE0
 #define TXEN_ TXEN0
 #define UBRR_ UBRR0
 #define UBRR_L UBRR0L
 #define UBRR_H UBRR0H
 #define UDRE_ UDRE0
 #define UCSR_A UCSR0A

 #define UCSR_C UCSR0C 

/* Initialisation */
void UART_Init(void)
{
	Transmitter_Ready = 1;

    UCSR_B = 0x00; //disable while setting baud rate
    UCSR_A = 0x00;
    UCSR_C = UCSRC_reg;
    UBRR_L = (unsigned char)(UBRR_reg & 0x00FF); //set baud rate lo
    UBRR_H = (unsigned char)(UBRR_reg >> 8); //set baud rate hi
    UCSR_B |= (1 << TXCIE_)|(1 << TXEN_)|(1 << RXCIE_)|(1 << RXEN_); 
	//UCSR_B &=~(1 << RXCIE_)|(1 << RXEN_);
	//0xD8; //  RXCIE_, TXCIE_, RXEN_, TXEN_ set to 1
}

/* Receiving part */

// UART1 Receive Complete Interrupt Function
SIGNAL(SIG_UART0_RECV)      
{
}

/* Transmit part */

// UART1 Transmit Complete Interrupt Function
SIGNAL(SIG_UART0_TRANS)
{   
    Transmitter_Ready = 1; 		// Indicate that the UART1 is now ready to send
}

//------------------------------------------------------
void UART_SendByte(char data_uart)
{
    //wait for UART to become available
	while(!Transmitter_Ready);
	//while(!(UCSR0A &(1<<UDRE0)));
    Transmitter_Ready = 0;
	
	UDR_ = data_uart;
}
//------------------------------------------------------

void UART_SendString(char *str) {
	while(*str) {
		UART_SendByte(*str); str++;
	}
}

void UART_SendBytes(char *str, int len) {
	while(len>0) {
		UART_SendByte(*str); str++; len--;
	}
}

void UART_SendInt(char i) {
	char s[10];
	snprintf(s, 10, "0x%02x", (char)i);
	UART_SendString(s);
}









